@@ -17,7 +17,7 @@ use crate::{Uri, basic_auth::BasicAuthExtractorError, utils};
 pub enum ErrorKind {
     /// Network error while handling request.
     /// This does not include erroneous status codes, `RejectedStatusCode` will be used in that case.
-    #[error("Network error")]
+    #[error("Network error: {analysis} ({error})", analysis=utils::reqwest::analyze_error_chain(.0), error=.0)]
     NetworkRequest(#[source] reqwest::Error),
     /// Cannot read the body of the received response
     #[error("Error reading response body: {0}")]
@@ -70,9 +70,9 @@ pub enum ErrorKind {
     #[error("Cannot find fragment")]
     InvalidFragment(Uri),
 
-    /// The given directory is missing a required index file
+    /// Cannot resolve local directory link using the configured index files
     #[error("Cannot find index file within directory")]
-    InvalidIndexFile(PathBuf),
+    InvalidIndexFile(Vec<String>),
 
     /// The given path cannot be converted to a URI
     #[error("Invalid path to URL conversion: {0}")]
@@ -100,9 +100,9 @@ pub enum ErrorKind {
     #[error("Cannot convert path '{0}' to a URI")]
     InvalidPathToUri(String),
 
-    /// Root dir must be an absolute path
-    #[error("Root dir must be an absolute path: '{0}'")]
-    RootDirMustBeAbsolute(PathBuf),
+    /// Invalid root directory given
+    #[error("Invalid root directory '{0}': {1}")]
+    InvalidRootDir(PathBuf, #[source] std::io::Error),
 
     /// The given URI type is not supported
     #[error("Unsupported URI type: '{0}'")]
@@ -158,10 +158,6 @@ pub enum ErrorKind {
     #[error("Error when using regex engine: {0}")]
     Regex(#[from] regex::Error),
 
-    /// Too many redirects (HTTP 3xx) were encountered (configurable)
-    #[error("Too many redirects")]
-    TooManyRedirects(#[source] reqwest::Error),
-
     /// Basic auth extractor error
     #[error("Basic auth extractor error")]
     BasicAuthExtractorError(#[from] BasicAuthExtractorError),
@@ -173,6 +169,15 @@ pub enum ErrorKind {
     /// Status code selector parse error
     #[error("Status code range error")]
     StatusCodeSelectorError(#[from] StatusCodeSelectorError),
+
+    /// Preprocessor command error
+    #[error("Preprocessor command '{command}' failed: {reason}")]
+    PreprocessorError {
+        /// The command which did not execute successfully
+        command: String,
+        /// The reason the command failed
+        reason: String,
+    },
 }
 
 impl ErrorKind {
@@ -182,33 +187,155 @@ impl ErrorKind {
     /// request type. The output is purely meant for humans (e.g. for status
     /// messages) and future changes are expected.
     #[must_use]
+    #[allow(clippy::too_many_lines)]
     pub fn details(&self) -> Option<String> {
         match self {
             ErrorKind::NetworkRequest(e) => {
-                // Get the relevant details from the specific reqwest error
-                let details = utils::reqwest::trim_error_output(e);
-
-                // Provide support for common error types
-                if e.is_connect() {
-                    Some(format!("{details} Maybe a certificate error?"))
-                } else {
-                    Some(details)
-                }
-            }
+                        // Get detailed, actionable error analysis
+                        Some(utils::reqwest::analyze_error_chain(e))
+                    }
             ErrorKind::RejectedStatusCode(status) => Some(
-                status
-                    .canonical_reason()
-                    .unwrap_or("Unknown status code")
-                    .to_string(),
-            ),
+                        status
+                            .canonical_reason()
+                            .unwrap_or("Unknown status code")
+                            .to_string(),
+                    ),
             ErrorKind::GithubRequest(e) => {
-                if let octocrab::Error::GitHub { source, .. } = &**e {
-                    Some(source.message.clone())
-                } else {
-                    None
-                }
-            }
-            _ => self.source().map(ToString::to_string),
+                        if let octocrab::Error::GitHub { source, .. } = &**e {
+                            Some(source.message.clone())
+                        } else {
+                            // Fall back to generic error analysis
+                            Some(e.to_string())
+                        }
+                    }
+            ErrorKind::InvalidFilePath(_uri) => Some(
+                "File not found. Check if file exists and path is correct".to_string()
+            ),
+            ErrorKind::ReadFileInput(e, path) => match e.kind() {
+                        std::io::ErrorKind::NotFound => Some(
+                            "Check if file path is correct".to_string()
+                        ),
+                        std::io::ErrorKind::PermissionDenied => Some(format!(
+                            "Permission denied: '{}'. Check file permissions",
+                            path.display()
+                        )),
+                        std::io::ErrorKind::IsADirectory => Some(format!(
+                            "Path is a directory, not a file: '{}'. Check file path",
+                            path.display()
+                        )),
+                        _ => Some(format!("File read error for '{}': {}", path.display(), e)),
+                    },
+            ErrorKind::ReadStdinInput(e) => match e.kind() {
+                        std::io::ErrorKind::UnexpectedEof => {
+                            Some("Stdin input ended unexpectedly. Check input data".to_string())
+                        }
+                        std::io::ErrorKind::InvalidData => {
+                            Some("Invalid data from stdin. Check input format".to_string())
+                        }
+                        _ => Some(format!("Stdin read error: {e}")),
+                    },
+            ErrorKind::ParseUrl(_, url) => {
+                        Some(format!("Invalid URL format: '{url}'. Check URL syntax"))
+                    }
+            ErrorKind::EmptyUrl => {
+                        Some("Empty URL found. Check for missing links or malformed markdown".to_string())
+                    }
+            ErrorKind::InvalidFile(path) => Some(format!(
+                        "Invalid file path: '{}'. Check if file exists and is readable",
+                        path.display()
+                    )),
+            ErrorKind::ReadResponseBody(error) => Some(format!(
+                "Failed to read response body: {error}. Server may have sent invalid data",
+            )),
+            ErrorKind::BuildRequestClient(error) => Some(format!(
+                "Failed to create HTTP client: {error}. Check system configuration",
+            )),
+            ErrorKind::RuntimeJoin(join_error) => Some(format!(
+                "Task execution failed: {join_error}. Internal processing error"
+            )),
+            ErrorKind::Utf8(_utf8_error) => Some(
+                "Invalid UTF-8 sequence found. File contains non-UTF-8 characters".to_string()
+            ),
+            ErrorKind::BuildGithubClient(error) => Some(format!(
+                "Failed to create GitHub client: {error}. Check token and network connectivity",
+            )),
+            ErrorKind::InvalidGithubUrl(url) => Some(format!(
+                "Invalid GitHub URL format: '{url}'. Check URL syntax",
+            )),
+            ErrorKind::InvalidFragment(_uri) => Some(
+                "Fragment not found in document. Check if fragment exists or page structure".to_string()
+            ),
+            ErrorKind::InvalidUrlFromPath(path_buf) => Some(format!(
+                "Cannot convert path to URL: '{}'. Check path format",
+                path_buf.display()
+            )),
+            ErrorKind::UnreachableEmailAddress(uri, reason) => Some(format!(
+                "Email address unreachable: '{uri}'. {reason}",
+            )),
+            ErrorKind::InvalidHeader(invalid_header_value) => Some(format!(
+                "Invalid HTTP header: {invalid_header_value}. Check header format",
+            )),
+            ErrorKind::InvalidBase(base, reason) => Some(format!(
+                "Invalid base URL or directory: '{base}'. {reason}",
+            )),
+            ErrorKind::InvalidBaseJoin(_) => Some("Check relative path format".to_string()),
+            ErrorKind::InvalidPathToUri(path) => match path {
+                path if path.starts_with('/') =>
+                    "To resolve root-relative links in local files, provide a root dir",
+                _ => "Check path format",
+            }.to_string().into(),
+            ErrorKind::InvalidRootDir(_, _) => Some(
+                "Check the root dir exists and is accessible".to_string()
+            ),
+            ErrorKind::UnsupportedUriType(uri_type) => Some(format!(
+                "Unsupported URI type: '{uri_type}'. Only http, https, file, and mailto are supported",
+            )),
+            ErrorKind::InvalidUrlRemap(remap) => Some(format!(
+                "Invalid URL remapping: '{remap}'. Check remapping syntax",
+            )),
+            ErrorKind::DirTraversal(error) => Some(format!(
+                "Directory traversal failed: {error}. Check directory permissions",
+            )),
+            ErrorKind::InvalidGlobPattern(pattern_error) => Some(format!(
+                "Invalid glob pattern: {pattern_error}. Check pattern syntax",
+            )),
+            ErrorKind::MissingGitHubToken => Some(
+                "GitHub token required. Use --github-token flag or GITHUB_TOKEN environment variable".to_string()
+            ),
+            ErrorKind::InsecureURL(uri) => Some(format!(
+                "Insecure HTTP URL detected: use '{}' instead of HTTP",
+                uri.as_str().replace("http://", "https://")
+            )),
+            ErrorKind::Channel(_send_error) => Some(
+                "Internal communication error. Processing thread failed".to_string()
+            ),
+            ErrorKind::InvalidUrlHost => Some(
+                "URL missing hostname. Check URL format".to_string()
+            ),
+            ErrorKind::InvalidURI(uri) => Some(format!(
+                "Invalid URI format: '{uri}'. Check URI syntax",
+            )),
+            ErrorKind::InvalidStatusCode(code) => Some(format!(
+                "Invalid HTTP status code: {code}. Must be between 100-999",
+            )),
+            ErrorKind::Regex(error) => Some(format!(
+                "Regular expression error: {error}. Check regex syntax",
+            )),
+            ErrorKind::BasicAuthExtractorError(basic_auth_extractor_error) => Some(format!(
+                "Basic authentication error: {basic_auth_extractor_error}. Check credentials format",
+            )),
+            ErrorKind::Cookies(reason) => Some(format!(
+                "Cookie handling error: {reason}. Check cookie file format",
+            )),
+            ErrorKind::StatusCodeSelectorError(status_code_selector_error) => Some(format!(
+                "Status code selector error: {status_code_selector_error}. Check accept configuration",
+            )),
+            ErrorKind::InvalidIndexFile(index_files) => match &index_files[..] {
+                [] => "No directory links are allowed because index_files is defined and empty".to_string(),
+                [name] => format!("An index file ({name}) is required"),
+                [init @ .., tail] => format!("An index file ({}, or {}) is required", init.join(", "), tail),
+            }.into(),
+            ErrorKind::PreprocessorError{command, reason} => Some(format!("Command '{command}' failed {reason}. Check value of the preprocessor option"))
         }
     }
 
@@ -271,9 +398,6 @@ impl PartialEq for ErrorKind {
             (Self::Regex(e1), Self::Regex(e2)) => e1.to_string() == e2.to_string(),
             (Self::DirTraversal(e1), Self::DirTraversal(e2)) => e1.to_string() == e2.to_string(),
             (Self::Channel(_), Self::Channel(_)) => true,
-            (Self::TooManyRedirects(e1), Self::TooManyRedirects(e2)) => {
-                e1.to_string() == e2.to_string()
-            }
             (Self::BasicAuthExtractorError(e1), Self::BasicAuthExtractorError(e2)) => {
                 e1.to_string() == e2.to_string()
             }
@@ -286,6 +410,7 @@ impl PartialEq for ErrorKind {
             (Self::InvalidBase(b1, e1), Self::InvalidBase(b2, e2)) => b1 == b2 && e1 == e2,
             (Self::InvalidUrlRemap(r1), Self::InvalidUrlRemap(r2)) => r1 == r2,
             (Self::EmptyUrl, Self::EmptyUrl) => true,
+            (Self::RejectedStatusCode(c1), Self::RejectedStatusCode(c2)) => c1 == c2,
 
             _ => false,
         }
@@ -325,7 +450,7 @@ impl Hash for ErrorKind {
             Self::InvalidBase(base, e) => (base, e).hash(state),
             Self::InvalidBaseJoin(s) => s.hash(state),
             Self::InvalidPathToUri(s) => s.hash(state),
-            Self::RootDirMustBeAbsolute(s) => s.hash(state),
+            Self::InvalidRootDir(s, _) => s.hash(state),
             Self::UnsupportedUriType(s) => s.hash(state),
             Self::InvalidUrlRemap(remap) => (remap).hash(state),
             Self::InvalidHeader(e) => e.to_string().hash(state),
@@ -337,10 +462,10 @@ impl Hash for ErrorKind {
                 std::mem::discriminant(self).hash(state);
             }
             Self::Regex(e) => e.to_string().hash(state),
-            Self::TooManyRedirects(e) => e.to_string().hash(state),
             Self::BasicAuthExtractorError(e) => e.to_string().hash(state),
-            Self::Cookies(e) => e.to_string().hash(state),
+            Self::Cookies(e) => e.hash(state),
             Self::StatusCodeSelectorError(e) => e.to_string().hash(state),
+            Self::PreprocessorError { command, reason } => (command, reason).hash(state),
         }
     }
 }
